
--[[ 
Passed this to main.lua.. Shouldn't cause any problems.

bump = require 'bump'
Timer = require 'timer'
state = require "stateswitcher"

player = {}
enemies = {}
move_rad= 100
anstable = {0,0}
clouds = {}
bullets = {}
bulletSpeed = 250
 
world = bump.newWorld()


function love.load()
  --comment out the below line to improve performance a bit. it allows for console comments via "print()".
  --io.stdout:setvbuf('no')
  
  love.window.setMode( 1100,600 )
  love.graphics.setBackgroundColor( 66, 66, 200 )
  
  score=0 
  Timer.every(1,function() score=score+1 end)-- raise it by 1 every second.
  
  enemy_counter= 0
  Timer.every(3,newEnemy) -- Spawn a new enemy every 3 seconds!
  
  Timer.every(1.5,enemyAI)
  
  player.x=0
  player.y=0
  player.width=32
  player.height=64
  player.name= "player"
  world:add(player, player.x,player.y,player.width,player.height)
  
  
end
--]]

--Filter for collision resolve.
--IE whether to pick "cross\slide\touch\bounce".
--item is what's being moved. Other is the object being moved into.
--NOTE THAT THIS FUNCTION SHOULD NOT BE USED FOR COLLISION RESOLUTIONS.
--That should be via the 'cols' table.
local collisionFilter = function(item, other)
  if (item.name == "bullet" and other.name == "player") then 
    return 'touch'
  elseif (item.name == "bullet" and other.name == "enemy") then 
    return 'cross'
  elseif (item.name == "player" and other.name == "bullet") then 
    return 'cross'
  elseif (item.name == "player" and other.name == "enemy") then 
    return 'slide'
  end
  --return nil
end

function love.update(dt)
  Timer.update(dt)
  teleEffect:update(dt)
  splat1:update(dt)
  for i,v in ipairs(bullets) do
    if (world:hasItem(v)) then
      local actualX, actualY, cols, len = world:move(v, v.x + (v.dx * dt), v.y + (v.dy * dt), collisionFilter)
      v.x = actualX
      v.y = actualY
      for j,col in ipairs(cols) do
        bulletReaction(col)
      end
      
      if (v.x<0 or v.x> love.graphics.getWidth() or v.y < 0 or v.y > love.graphics.getHeight()) then    
        world:remove(bullets[i])
        table.remove(bullets,i)
      end
    else table.remove(bullets,i) 
    end
  end
end

function love.draw()
  love.graphics.setColor(255,255,255)
  
  --love.graphics.setColor(0,0,0)
  love.graphics.setFont(fontDef,40)
  love.graphics.print("score = " ..score,love.graphics.getWidth( )/2-60,20 )
  
  --draw player
  love.graphics.draw(playerpic,player.x,player.y)
  --[[
  love.graphics.rectangle('fill',player.x,player.y,player.width,player.height)
  love.graphics.setColor(255,0,0)
  love.graphics.rectangle('fill',player.x+8,player.y+16,player.width/2,player.height/2)
  love.graphics.setColor(255,255,255)
  --]]
  for i, v in ipairs(enemies) do
    if v.class=='A' then
      love.graphics.draw(enemyApic,v.x,v.y)
    elseif v.class=='B' then
      love.graphics.draw(enemyBpic,v.x,v.y)
    elseif v.class=='C' or v.class=='D'  then
      love.graphics.draw(enemyCpic,v.x,v.y)
    end
    
    --love.graphics.rectangle('fill',v.x,v.y,v.width,v.height)
  end
  
  for i,v in ipairs(bullets) do
		--love.graphics.rectangle("fill", v.x, v.y, 6,6)
    love.graphics.draw(shotpic,v.x-6,v.y-6)
	end
  
  love.graphics.draw(teleEffect)
  love.graphics.draw(splat1)
  
end

function love.keypressed(key)
  
   if key == "escape" then
     love.event.quit()
     
   elseif key =="space" then -- teleport
     if tele_okay == false then return end
     anstable=getcircleCoor(love.mouse.getX(),love.mouse.getY())
     teleEffect:moveTo(player.x+16,player.y+32)
     --Timer.every(0.35, function() teleEffect:emit(1) end,3)
     teleEffect:emit(1)
     local actualX, actualY, cols, len = world:move(player, anstable[1], anstable[2],collisionFilter)
     player.x , player.y = actualX, actualY
     teleEffect:moveTo(player.x+16,player.y+32)
     teleEffect:emit(1)
     tele_okay = false
     Timer.after(0.5,function() tele_okay = true end)
    
   
   end
   
end

-- This calculates the new position of the character.
function getcircleCoor(newX,newY)
  local x1,y1 = player.x,player.y
  local x2,y2 = newX,newY
  local r = move_rad
  local d = math.sqrt((x1-x2)^2+(y1-y2)^2)
  
  local results = {}
  results[1] = x1+r*(x2-x1)/d -- final X value on the circle
  results[2] = y1+r*(y2-y1)/d -- final Y value on the circle
  
  return results
end

-- Create an enemy and put him in a random place on the map.
-- According to the 'enemy timer' variable, create tougher versions of enemy.
-- Enemy "Class" is  'A'/'B'/'C'/'D'
-- 'A' = normal shots
-- 'B' = quick shots
-- 'C' = Plasma beam
-- 'D' = Plasma beam and higher HP
function newEnemytst(class)
  x,y = love.math.random(0,1068),love.math.random(0,536)
  enemy_counter = enemy_counter+1
  local newEnemy ={x=x,y=y,width=32, height = 64,name = "enemy", id = enemy_counter, class = class}
  table.insert(enemies,newEnemy)
  world:add(newEnemy, newEnemy.x,newEnemy.y,newEnemy.width,newEnemy.height)
end

function newEnemy()
  x,y = love.math.random(0,1068),love.math.random(0,536)
  enemy_counter = enemy_counter+1
  local class
  if (enemy_counter < 4) then
    class = 'A'
  elseif (enemy_counter < 8) then
    class = 'B'
  elseif (enemy_counter < 12) then
    class = 'C'
  else
    class = 'D'
  end
  local newEnemy ={x=x,y=y,width=32, height = 64,name = "enemy", id = enemy_counter, class = class}
  table.insert(enemies,newEnemy)
  world:add(newEnemy, newEnemy.x,newEnemy.y,newEnemy.width,newEnemy.height)
end

-- You should give a delay to some enemies , with the "wait" function of the timer, if you wanna. Give a random "wait".
function enemyAI()
  for i, v in ipairs(enemies) do
        local startX = v.x + v.width / 2
        local startY = v.y + v.height / 2
        local endX = player.x+16
        local endY = player.y+32
        local angle = math.atan2((endY - startY), (endX - startX))
        --The following lines should be based on the enemy. Tougher enemies shoot tougher bullets.
        if v.class == 'A' then
          local bulletDx = bulletSpeed * math.cos(angle)
          local bulletDy = bulletSpeed * math.sin(angle)
          local newBullet =  {x = startX, y = startY, dx = bulletDx, dy = bulletDy,name = "bullet", id = v.id, class = v.class}
          table.insert(bullets, newBullet)
          world:add(newBullet, newBullet.x,newBullet.y,3,3)
        elseif v.class == 'B' then
          local bulletDx = 2*bulletSpeed * math.cos(angle)
          local bulletDy = 2*bulletSpeed * math.sin(angle)
          local newBullet =  {x = startX, y = startY, dx = bulletDx, dy = bulletDy,name = "bullet", id = v.id, class= v.class}
          table.insert(bullets, newBullet)
          world:add(newBullet, newBullet.x,newBullet.y,3,3)
        elseif v.class == 'C' or 'D' then
          local bulletDx = 2*bulletSpeed * math.cos(angle)
          local bulletDy = 2*bulletSpeed * math.sin(angle)
          Timer.every(0.03,function()
            local newBullet =  {x = startX, y = startY, dx = bulletDx, dy = bulletDy,name = "bullet", id = v.id, class = v.class}
            table.insert(bullets, newBullet)
            world:add(newBullet, newBullet.x,newBullet.y,3,3)
            end
          ,10)
          
          
        end
        
      end
end

-- Gets a collision and reacts to it appropriately. Remember, each "col" is a table with info about the collision.
function bulletReaction(col)
  local i
  if col.item.name=="bullet" then
    if col.other.name == "player" then --player is hit. Game over!
      final_score = score
      state.switch ("gameover")
      --[[ This resets the game loop.
      score = 0
       enemy_counter = 0
       tablePurge(enemies)
       tablePurge(bullets)  
       --]]
    elseif ((col.item.class == 'C' or col.item.class == 'D') and col.item.id ~= col.other.id) then
      for i,v in ipairs(enemies) do
        if col.other.id == v.id then
          splat1:moveTo( v.x+16, v.y+32 )
          splat1:emit(6)
          table.remove(enemies,i)
        end
      end
      world:remove(col.other)
    elseif (col.other.name == "enemy" and col.item.id ~= col.other.id )then -- Enemy is hit. Find enemy and bullet position and delete.
      
      for i,v in ipairs(enemies) do
        if col.other.id == v.id then
          splat1:moveTo( v.x+16, v.y+32 )
          splat1:emit(6)
          table.remove(enemies,i)
        end
      end
      ---[[
      for i,v in ipairs(bullets) do
        if col.item.id == v.id then
          table.remove(bullets,i)
        end
      end
      --]]
      world:remove(col.other)
      world:remove(col.item)
    end
  end
  
end

function love.mousepressed(x,y,key)
end